/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

/*
	THis file holds class declarations and global functions, it is included in every header file
*/

#ifndef __MONTAGEINTERFACE_H__
#define __MONTAGEINTERFACE_H__

#ifndef ArmeBasicTypes_h
#include "ArmeBasicTypes.h"
#endif //ArmeBasicTypes_h

#ifndef MEMixingInfo_h
#include "MEMixingInfo.h"
#endif //MEMixingInfo_h

#define GET_VALUE(a) (a.data)
#define USE_CACHED_IMAGES (YES)

struct MIVPos
{
	SInt32 data;
	
	bool operator==(const MIVPos& arg2) const {return data == arg2.data;};
	bool operator!=(const MIVPos& arg2) const {return data != arg2.data;};
	bool operator<(const MIVPos& arg2) const {return data < arg2.data;};
	bool operator>(const MIVPos& arg2) const {return data > arg2.data;};
	bool operator<=(const MIVPos& arg2) const {return data <= arg2.data;};
	bool operator>=(const MIVPos& arg2) const {return data >= arg2.data;};
	MIVPos& operator +=(const MIVPos& arg) {data += arg.data; return *this;};
	MIVPos& operator -=(const MIVPos& arg) {data -= arg.data; return *this;};
	MIVPos& operator =(const MIVPos& arg) {data = arg.data; return *this;};
	MIVPos& operator *=(const int& arg) {data *= arg; return *this;};
};

inline std::ostream& operator<<(std::ostream& s, const MIVPos& vpos)
{
	s << vpos.data;
	return s;
}

inline MIVPos operator+(const MIVPos& arg1, const MIVPos& arg2) {MIVPos res; res.data = arg1.data + arg2.data; return res;};
inline MIVPos operator-(const MIVPos& arg1, const MIVPos& arg2) {MIVPos res; res.data = arg1.data - arg2.data; return res;};
inline MIVPos operator*(const MIVPos& arg1, const int& arg2) {MIVPos res; res.data = arg1.data * arg2; return res;};
inline MIVPos operator*(const int& arg2, const MIVPos& arg1) {MIVPos res; res.data = arg1.data * arg2; return res;};

inline MIVPos _MIVPos(SInt32 val)
{
	MIVPos vpos;
	GET_VALUE(vpos) = val;
	return vpos;
}

#define VPOS_UNDEF	_MIVPos(-2147483647)
#define VPOS_MIN	_MIVPos(-2147483646)
#define VPOS_MAX	_MIVPos(2147483647)
#define VPOS_ZERO	_MIVPos(0)

#define SCALABLE_TITLEBAR
//#define SMALL_TITLEBAR
//#define DRAW_HANDLES
#define FOOTER_EXISTS


#ifdef SCALABLE_TITLEBAR
extern const MIVPos		kRepTitleHeight;
extern const MIVPos		kRepFooterHeight;
extern const MIVPos		kRepHeaderHeight;
extern const MIVPos    	kRepTitleBarHeight;
#else
extern const int		kRepTitleHeight;
extern const MIVPos		kRepFooterHeight;
extern const int    	kRepTitleBarHeight;
#endif

extern const MIVPos		kBetweenClipsHeight;
extern const int		kRepButtonHitTolerance;

#if defined(__MWERKS__)
#define kCacheingMinSize (NSMakeSize(1500,512))
#else //#if defined(__MWERKS__)
extern const NSSize 	kCacheingMinSize;
#endif //#if defined(__MWERKS__)

extern const float		kCacheingHorizontalExtra;
extern const float		kCacheingVerticalExtra;
extern const MIVPos		kRepAutomationCurveHeight;


struct MIPoint
{
	ArmeTime	time;
	MIVPos		vpos;
};

struct MISize
{
	ArmeTime	duration;
	MIVPos		height;
};

struct MIRect
{
	MIPoint	origin;
	MISize	size;
};

inline std::ostream& operator<<(std::ostream& s, const MIRect& rc)
{
	s << "origin.time= " << rc.origin.time << " origin.vpos=" << rc.origin.vpos << " size.duration=" << rc.size.duration << " size.height=" << rc.size.height << " ";
	return s;
}

extern const MIRect MIZeroRect;
extern const MISize MIZeroSize;
extern const MIPoint MIZeroPoint;

enum MIRectEdge
{
	MIMinXEdge = 1,
	MIMinYEdge = 2,
	MIMaxXEdge = 4,
	MIMaxYEdge = 8
};


@class MIImageController;
extern MIImageController *mainImageController;

MIRect	MIMakeRect(ArmeTime x, MIVPos y, ArmeTime w, MIVPos h);
MIPoint	MIMakePoint(ArmeTime x, MIVPos y);
MISize	MIMakeSize(ArmeTime length, MIVPos height);
ArmeTime MIMaxX(MIRect aRect);
MIVPos MIMaxY(MIRect aRect);
ArmeTime MIMinX(MIRect aRect);
MIVPos MIMinY(MIRect aRect);
ArmeTime MIWidth(MIRect aRect);
MIVPos MIHeight(MIRect aRect);
BOOL MIContainsRect( MIRect rect1, MIRect rect2 );
MIRect MIUnionRect( MIRect rect1, MIRect rect2 );
BOOL MIEqualRects( MIRect rect1, MIRect rect2 );
BOOL MIEqualPoints( MIPoint p1, MIPoint p2);
BOOL MIEqualSizes( MISize s1, MISize s2 );
MIPoint TranslateMIPoint( MIPoint p1, MIPoint p2 );

ArmeTime	MIArmeTimeFromString(NSString* time);
NSString*	MIStringFromArmeTime(ArmeTime time);

NSRect			ClipNSRect( NSRect rect, NSRect clip );
NSRect			TranslateNSRect( NSRect rect, NSPoint vector );
NSRect			ScaleNSRect(NSRect rect, float hZoom, float vZoom );
NSPoint			TranslateNSPoint( NSPoint origin, NSPoint vector );
NSRect			RectWithPoints(NSPoint point1, NSPoint point2);
BOOL			IsNSPointInRect(NSPoint point, NSRect rect);
Rect 			NSRect2Rect(const NSRect& inNSRect);
// wrapper to Carbon ::MapRect() function
NSRect 			MapNSRect(const NSRect& nsr, const NSRect& srcNSRect, const NSRect& dstNSRect);
// Scales one NSRect to fit inside another maintaining aspect ratio
NSRect 			ScaleNSRectToNSRect(const NSRect& inSourceRect, const NSRect& inDestRect, bool inCenter);

id			CreatePalette( NSPoint position, NSRect dockableItemSize, NSString *title );

NSColor		*NegativeOfColor( NSColor *color);

@class MIWSSelection;
MEVAORouting MIGetSelectionVAORouting(MIWSSelection* selection);

#endif